package alignment;

import ModellingTool.MainMenu;
import ModellingTool.RunParameters;
import com.sun.xml.internal.ws.util.UtilException;
import meshi.util.crossLinking.MySequence;
import meshi.util.crossLinking.MySequenceList;
import UtilExceptions.*;

import javax.swing.*;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.*;
import java.util.concurrent.*;
import java.util.function.Predicate;
import java.util.stream.Collectors;

public class RvalAlignerCluster extends SwingWorker<Void, Integer> {
	
	private final String profileFilePath;
	private final String fastaSeqPath;
	private final boolean fullFasta;
	private final String swissProtPath;
	private final Character[] symmetricChains;
	private final char chainToProc;
	private final String pdbCode;
	private final int threadNum;
	private final boolean silent;
	private final int num_better;
	private final String[] fastaSeq;
	//	private final CustomOutputStream customOut;
		private String row;
	//	private final logObj log;
	private RvalSequence sourceSeq;
	private JProgressBar progressBar;
	private ExecutorService executor;
	private static DecimalFormat df6 = new DecimalFormat(".######");
	private static DecimalFormat df2 = new DecimalFormat(".##");

	/**
	 * This ctor creates a worker thread that wraps around the NW aligner method by Nir Kalisman and provides input/output arguments to allow output
	 * other than STD.
	 *
	 * @param profileFilePath path to profile for threading
	 * @param params          params object
	 */
	public RvalAlignerCluster(String profileFilePath, RunParameters params) {
		String[] fastaSeq1;
		this.profileFilePath = profileFilePath;
		this.fastaSeqPath = params.getFastaFile();
		this.fullFasta = params.getFullFasta();
		try{
			fastaSeq1 = params.getFastaSeq().split("\n");
		} catch (Exception e){
			fastaSeq1 = null;
		}
		this.fastaSeq = fastaSeq1;
		this.symmetricChains = params.getSymmetricHomologues();
		this.chainToProc = params.getChainToProcess();
		this.swissProtPath = params.getSWISSProt().getAbsolutePath();
		this.pdbCode = params.getPDBcode();
		this.threadNum = params.getThreadLimit();
		this.silent = params.getSilent();
		this.num_better = params.getNumOfBetterThreads();
		//		customOut = params.getCustomOut();
//		log = new logObj();

		row = "PDB, Symmetric Chains, Chain length, Structural Gaps, Mean Z-Score, Threading Score, Quality Index,  " +
				"Entries with higher score , Percentile against UniProt, Number of UniProt Sequences, Similarity to PDB threading \n";
	}

	private static double singleRvalAlignerRun(String seq, RvalSequence Rseq) {
		AminoAcidSequence AAseq = new AminoAcidSequence(seq);
		NeedlemanWunchSolver NW = new NeedlemanWunchSolver(Rseq, AAseq, new RvalScoringScheme());
		//		if (toPrintAlignment) {
		//			NW.printAlignment();
		//
		//			log.logString += NW.logAlignment();
		//		}
		NW.printAlignment();
		return NW.alignmentScore();
	}


	@Override
	protected Void doInBackground() throws Exception {
		System.out.flush();
		MySequenceList swissProt = new MySequenceList(swissProtPath);
		MySequenceList mySeqList = null;
		System.out.println("Total num of SwissProt seqs: " + swissProt.size() + "\n");
		if (fastaSeqPath != null){
			mySeqList = new MySequenceList(fastaSeqPath);
		} else if (fastaSeq != null){
			mySeqList = new MySequenceList(fastaSeq);
		} else{
			throw new Exception("Must have either fasta sequence or file");
		}
		int trueLength = mySeqList.get(0).seq().length();
		System.out.flush();
		sourceSeq = new RvalSequence(profileFilePath);    // txt version of CSV "allWithSeq"
		int srcLength = sourceSeq.length();
//		log.logString += "working on profile file: " + profileFilePath + "\n";
		System.out.println("working on profile file: " + profileFilePath + "\n");
//		log.logString += mySeqList.fileName().substring(mySeqList.fileName().lastIndexOf("/") + 1, mySeqList.fileName()
//				.lastIndexOf(".fasta")) + ":   ";
//		log.logString += "Structure positions: " + sourceSeq.size() + "   Length of *true* seq: " + mySeqList.get(0).seq().length() +
//				"\n---------------------------------------------------------------------------------------\n";
		System.out.println(mySeqList.fileName().substring(mySeqList.fileName().lastIndexOf("/") + 1, mySeqList.fileName().lastIndexOf(
				".fasta")) + ":   Structure positions: " + sourceSeq.size() + "   Length of *true* seq: " + trueLength + "\n---------------------------------------------------------------------------------------\n");
		AminoAcidSequence AAseq;
		try {
			AAseq = new AminoAcidSequence(mySeqList.get(0).seq());
		} catch (RuntimeException e) {
			System.out.println("Error: ");
			System.out.println(e.getMessage());
			throw e;
		}
		NeedlemanWunchSolver NWTrueSeq = new NeedlemanWunchSolver(sourceSeq, AAseq, new RvalScoringScheme());
		double scoreConstBonus = (0.25 * ( trueLength - srcLength) );
		NWTrueSeq.setScoreConstBonus(scoreConstBonus);
		NWTrueSeq.printAlignment();
//		double sameSeqScore = singleRvalAlignerRun(mySeqList.get(0).seq(), sourceSeq);
		double sameSeqScore = NWTrueSeq.alignmentScore();

		String similarity = df2.format(NWTrueSeq.bestAlign.matches * 100f / sourceSeq.length() );
		double trueQindx = sameSeqScore / sourceSeq.length();
		System.out.println("Quality Index: " + trueQindx + " and alignment similarity: " + similarity);
		StringBuilder sb = new StringBuilder();

		sb.append(chainToProc);
		for (Character c : symmetricChains){
			sb.append((char)c);
		}
		row += pdbCode+ ", ";
		row += sb.toString();
		row += ", " + sourceSeq.size() + ", " + NWTrueSeq.numGaps + ", " + ", " + sameSeqScore +", " + trueQindx + ", ";

//		row += fastaSeqPath.substring(0, fastaSeqPath.indexOf(".fasta")) + ",";
//		row += sourceSeq.size() + ", " + mySeqList.get(0).seq().length() + ", " + df6.format(sameSeqScore) + ", " + NWTrueSeq.bestAlign.matches;
		System.out.flush();

		if (fullFasta) {
			
//			log.logString += "\nProcessing SwissProt sequences:\n";
			System.out.println("\nProcessing SwissProt sequences:\n");
			ExecutorService threadingExec = new ThreadPoolExecutor(20,20, 5000L, TimeUnit.MILLISECONDS, new LinkedBlockingQueue<>());
//			ExecutorCompletionService<Double[]> executorThreads = new ExecutorCompletionService<>(threadingExec);
			Queue<Future<Double[]>> futures = new ConcurrentLinkedDeque<>();
			int validSwissProt = 0;

			for (int ccc = 0; ccc < swissProt.size(); ccc++) {
				if (silent){
					if (ccc % 1024 == 0) {
//						setProgress(Math.round((float)ccc / swissProt.size() * 100f));
						System.out.print("validated "+ccc + " UNIPROT sequences \r");
					}
				}
//								if (ccc % 65536 == 0) {
//									setProgress(Math.round((float)ccc / swissProt.size() * 100f));
//									System.out.print(ccc + ".. ");
//									publish();
//								}
				//				if (ccc % 8192 == 0) {
				//					setProgress(Math.round((float)ccc / swissProt.size() * 100f));
				////					System.out.print(ccc + ".. ");
				//					publish();
				//				}
				MySequence mySeq = swissProt.get(ccc);
				if (!(mySeq.seq().indexOf('X') > -1) &&
						!(mySeq.seq().indexOf('B') > -1) &&
						!(mySeq.seq().indexOf('J') > -1) &&
						!(mySeq.seq().indexOf('Z') > -1) &&
						!(mySeq.seq().indexOf('O') > -1) &&
						!(mySeq.seq().indexOf('U') > -1)) {
					if (mySeq.seq().length() >= sourceSeq.size()) {
						final int finalCcc = ccc;
						validSwissProt++;
						if (validSwissProt % 1024 == 0) {
							progressBar.setString(validSwissProt + " valid seqs found...");
						}
							futures.add(threadingExec.submit(() -> {
							AminoAcidSequence targetSeq = new AminoAcidSequence(mySeq.seq());
							NeedlemanWunchSolver NW = new NeedlemanWunchSolver(sourceSeq, targetSeq, new RvalScoringScheme());
							NW.setScoreConstBonus(scoreConstBonus);
							NW.logAlignment();

							double tempQindx = NW.alignmentScore() / sourceSeq.length();
							return new Double[]{NW.alignmentScore(), (double) finalCcc, tempQindx};
						}));
					}
				}

			}
//			threadingExec.shutdown();
			progressBar.setString(validSwissProt + " valid seqs found.");
			if (silent){
				System.out.println(validSwissProt + " valid seqs found.");
			}
			progressBar.setMaximum(validSwissProt);
			int counter = 0;
			Future<Double[]> completedThread;
			List<Double[]> resultsList = new LinkedList<>();
			while (counter < validSwissProt) {
				completedThread = futures.poll();
				counter++;

				resultsList.add(completedThread.get());
//				if (counter % 512 == 0) {
					setProgress(Math.round((float)(counter) / validSwissProt * 100f));
//					System.out.print(counter + ".. ");
					publish(counter);
//				}
//				if (counter % 65536 == 0) {
//					setProgress(Math.round((float)(counter) / validSwissProt * 100f));
//					System.out.println(counter + ".. ");
//					publish();
//				}
				if (silent){
					if (counter % 128 == 0){
						System.out.print("processed " + counter + " sequences of " + validSwissProt + "            \r");
					}
				}
			}

			setProgress(Math.round((float)(counter) / validSwissProt * 100f));
			publish(counter);
			
			Collections.sort(resultsList,new ResultsComparator());
			List<Double[]> validList = resultsList.stream().filter(p->p[0]>-1000).collect(Collectors.toList());
			List<Double[]> betterList = validList.stream().filter(FilterPredicates.largerThan(sameSeqScore)).collect(Collectors.toList());
			Collections.sort(betterList,new ResultsComparator());
			Collections.sort(validList,new ResultsComparator());
			double largest = validList.get(0)[0];
			int validSeqCounter = validList.size();
			int betterScoringSeqs = betterList.size();
			double percent = betterScoringSeqs * 100f / validSeqCounter ;

			
			row += betterScoringSeqs+1 + ", " + df2.format(percent) + ", " + validSeqCounter + ", ";
//			System.out.println();
//			log.logString += betterScoringSeqs + " entries out of valid " + validSeqCounter + " are with better score than the *true*. The " +
//					"best score that is not the *true* sequence is: " + largest + "\n";
			System.out.println(betterScoringSeqs + " entries out of valid " + validSeqCounter + " are with better score than the *true*. The " +
					"best score that is not the *true* sequence is: " + largest);
			
			System.out.println("The top "+ num_better +" best scores are:");
			for (int i=0; i<num_better; i++){
				MySequence mySeq = swissProt.get(validList.get(i)[1].intValue());
				System.out.println("Number " + (i+1) +" is: " + mySeq.title().substring(0, Math.min(mySeq.title().length() - 1, 60))
									+ "\nWith Quality Index of: " + validList.get(i)[2]);
				singleRvalAlignerRun(mySeq.seq(), sourceSeq);
				System.out.println();
				System.out.flush();
			}
		} else{
			row +=  " , " +  " , " + "  , " ;
		}

		row += similarity + "\n";
		File resultCSV = new File(profileFilePath.replace(".txt", "_ThreadResults.csv"));
//		File logFile = new File(profileFilePath.replace(".txt", "_ThreadLog.txt"));

		FileWriter FW;
		try {
			FW = new FileWriter(resultCSV);
			FW.write(row);
			FW.close();

		} catch (IOException e) {
			e.printStackTrace();
		}
		MainMenu.doneThreading = true;
		publish(progressBar.getMaximum());
		setProgress(progressBar.getMaximum());
		System.out.flush();
		return null;
	}

	@Override
	protected void process(List<Integer> s) {
//		for (String line : s) {
//			System.out.print(line);
//		}
		progressBar.setValue(s.get(s.size()-1));
		progressBar.setString(progressBar.getValue() +" of " +progressBar.getMaximum() + " done");
		if (progressBar.getValue() == progressBar.getMaximum()){
			System.out.println("Finished threading all results.");
			System.out.flush();

		}
	}

	@Override
	protected void done() {
		
	}

	public void setProgressBar(JProgressBar progressBar) {
		this.progressBar = progressBar;
	}

	public void setExecutor(ExecutorService executor) {
		this.executor = executor;
	}

//	private static class logObj {
//		public String logString = "";
//
//		public logObj() {
//		}
//
//	}
//
	
	
	
}

class ResultsComparator implements Comparator<Double[]>
{
	
	@Override
	public int compare(Double[] o1, Double[] o2) {
		return o2[0].compareTo(o1[0]);
	}
}

class FilterPredicates {
	public static Predicate<Double[]> largerThan(Double trueRes){
		return p -> p[0] > trueRes;
	}
}